***************************************************
*                                                 *
* RATS Program name: simulate.prg                 *
*                                                 *
* Author:  Anders Warne, anders.warne@riksbank.se *
*                                                 *
* Date:    September, 2000                        *
*                                                 *
* Purpose: Generate data from a 2-state MS-VAR    *
*          model with 2 lags and test for         *
*          Granger noncausality in a VAR(12)      *
*          model for the levels and a VAR(11)     *
*          model for the first differences.       *
*                                                 *
***************************************************
*
calender 1959 1 12
allocate 0 1995:2
*
open data ymm5995.asc
data(format=free,org=obs) 1959:1 1995:2 income money
*
set dlny 1959:2 1995:2 = 1200*log(income/income{1})
set dlnm 1959:2 1995:2 = 1200*log(money/money{1})
*
set lny 1959:1 1995:2 = 1200*log(income)
set lnm 1959:1 1995:2 = 1200*log(money)
*
*graph 1
*# dlny
*
*graph 1
*# dlnm
*
dec rect a1(2,4) a2(2,4) c1(2,1) c2(2,1) omemasq1 omegasq2
dec symm omega1(2,2) omega2(2,2)
dec int iter laglev lagdiff
dec real p11 p22
*
comp iter = 10000
comp laglev = 12
comp lagdiff = laglev-1
*
*************************************
*                                   *
* Inputting parameters for the DGP  *
*                                   *
*************************************
*
input a1
  .447 .378 -.084 -1.67
  .067 .232 -.031 -.149
*
input c1
  -1.006
  4.742
*
input a2
  .274 -.104  .127 .274
 -.025  .476 -.064 .203
*
input c2
  1.648
  2.410
*
input omega1
  242.546
  -19.168 55.940
*
input omega2
  46.535
   2.872 14.483
*
comp p11 = .744
comp p22 = .914
comp pi1 = .251
*
comp omegasq1=%decomp(omega1)
comp omegasq2=%decomp(omega2)
*
make(trans) y 1959:2 1959:3 initval
# dlny dlnm
*
make(trans) xma 1959:1 1959:1 blah
# lny lnm
*
*************************************************
*                                               *
* Determine the number of observation from nobs *
*                                               *
*************************************************
*
make(trans) ydata 1959:4 1995:2 nobs
# dlny dlnm
*
make(trans) xdata 1959:1 1995:2 nobs2
# lny lnm
*
**************************************************
*                                                *
* Starting the iterations to generate iter # of  *
* samples.                                       *
*                                                *
**************************************************
*
dec rect regimes(1,nobs) probreg(1,nobs) newobs(2,1)
dec rect innov(2,nobs) oldval(4,1) ymat xmat innov1 vcvchol
dec rect vecres(2,1) simf(iter,2)
dec symm vcvinnov
dec int vecobs
dec real level diff
dec vect[series] dyseries dmseries yseries mseries
*
dim ymat(2,(nobs+2))
dim xmat(2,(nobs+3))
dim innov1(2,(nobs+50))
dim dyseries(1) dmseries(1) yseries(1) mseries(1)
*
comp simf=%const(0)
*
infobox(action=define,progress,lower=1,upper=iter)
*
do i=1,iter
*
****************************************************
*                                                  *
* Determine the regime from a uniform distribution *
* with the initial regime given by state 1 if      *
* the first draw in probreg is less than pi1 and   *
* by state 2 otherwise                             *
*                                                  *
****************************************************
*
   comp probreg=%uniform(0,1)
   do j=1,nobs
      if j==1
         if probreg(1,1)<pi1
            comp regimes(1,1)=1
         else
            comp regimes(1,1)=2
      else
         if regimes(1,j-1)==1
            if probreg(1,j)<p11
               comp regimes(1,j)=1
            else
               comp regimes(1,j)=2
         else
            if probreg(1,j)<p22
               comp regimes(1,j)=2
            else
               comp regimes(1,j)=1
   end do j
*
   comp newobs=%const(0)
   comp innov=%const(0)
   comp innov1=%const(0)
   comp ymat=%const(0)
   comp xmat=%const(0)
*
   do i1=1,2
      do j1=1,2
         comp ymat(i1,j1)=y(i1,j1)
      end do j1
   end do i1
*
   do i1=1,2
      comp xmat(i1,1)=xma(i1,1)
   end do i1
*
   comp innov1=%ran(1)
*
   do i1=1,2
      do j1=1,nobs
         comp innov(i1,j1)=innov1(i1,j1+50)
      end do j1
   end do i1
*
   comp vcvinnov=(1.0/float(nobs))*innov*tr(innov)
   comp vcvchol=%decomp(vcvinnov)
   comp innov=inv(vcvchol)*innov
*
   do j=1,nobs
      comp oldval=%const(0)
      do i1=1,2
         do j1=1,2
            comp vecobs=j1+(2*(i1-1))
            comp oldval(vecobs,1)=ymat(j1,j+2-i1)
         end do j1
      end do i1
      do j1=1,2
         comp vecres(j1,1)=innov(j1,j)
      end do j1
      if regimes(1,j)==1
         {
         comp newobs=c1+(a1*oldval)+(omegasq1*vecres)
         }
      else
         {
         comp newobs=c2+(a2*oldval)+(omegasq2*vecres)
         }
      do j1=1,2
         comp ymat(j1,j+2)=newobs(j1,1)
      end do j1
   end do j
*
***************************************************
*                                                 *
* The new observations on first differences have  *
* now been determined                             *
*                                                 *
***************************************************
*
   set dyseries(1) 1959:2 1995:2 = ymat(1,t-1)
   set dmseries(1) 1959:2 1995:2 = ymat(2,t-1)
*
***************************************************
*                                                 *
* To set up the levels observations we need to    *
* accumulate the first differences and then       *
* add the initial value from xma                  *
*                                                 *
***************************************************
*
   do i1=1,2
      do j1=1,(nobs+2)
         comp xmat(i1,j1+1)=ymat(i1,j1)+xmat(i1,j1)
      end do j1
   end do i1
*
   set yseries(1) 1959:1 1995:2 = xmat(1,t)
   set mseries(1) 1959:1 1995:2 = xmat(2,t)
*
   linreg(noprint) yseries(1)
   # constant yseries(1){1 to 12} mseries(1){1 to 12}
   exclude(noprint)
   # mseries(1){1 to 12}
   comp level=%cdstat
*
   linreg(noprint) dyseries(1)
   # constant dyseries(1){1 to 11} dmseries(1){1 to 11}
   exclude(noprint)
   # dmseries(1){1 to 11}
   comp diff=%cdstat
*
   comp simf(i,1)=level
   comp simf(i,2)=diff
*
   dis(store=info) 'LF=' level ', DF=' diff
   infobox(current=i) info

end do i
*
infobox(action=remove)
*
messagebox 'simulate.prg has finished'
*
default write(format='(10F6.3)')
write simf
*
